@@ -169,9 +169,11 @@
   (sb-thread::init-main-thread)
 
   ;; not sure why this is needed on some architectures. Dark magic.
+  #-linkage-space
   (setf (fdefn-fun (find-or-create-fdefn '%coerce-callable-for-call))
         #'%coerce-callable-to-fun)
   (show-and-call !loader-cold-init)
+  #+linkage-space (show-and-call sb-vm::!initialize-lisp-linkage-table)
   ;; Assert that FBOUNDP doesn't choke when its answer is NIL.
   ;; It was fine if T because in that case the legality of the arg is certain.
   ;; And be extra paranoid - ensure that it really gets called.
