@@ -891,9 +891,7 @@
     (unless (csubtypep (tn-ref-type args) (specifier-type 'symbol))
       (test-type value temp (if not-p target not-target) t (symbol-widetag)
                  :value-tn-ref args))
-    (inst ldrh temp (@ value (+ (ash symbol-name-slot word-shift)
-                               (- other-pointer-lowtag)
-                               6)))
+    (inst ldrh temp (@ value (- 2 other-pointer-lowtag)))
     (inst cmp temp sb-impl::+package-id-keyword+)
     (inst b (if not-p :ne :eq) target)
     not-target))
