@@ -114,21 +114,6 @@ struct page {
     generation_index_t gen;
 };
 extern struct page *page_table;
-extern page_index_t page_table_pages;
-
-/* Find the page index within the page_table for the given
- * address. Return -1 on failure. */
-static inline page_index_t find_page_index(void *addr)
-{
-    if (addr >= (void*)DYNAMIC_SPACE_START) {
-        page_index_t index = ((uintptr_t)addr -
-                              (uintptr_t)DYNAMIC_SPACE_START) / GENCGC_PAGE_BYTES;
-        if (index < page_table_pages)
-            return (index);
-    }
-    return (-1);
-}
-extern char *page_address(page_index_t);
 
 /* New objects are allocated to PAGE_TYPE_MIXED or PAGE_TYPE_CONS */
 /* If you change these constants, then possibly also change the following
@@ -564,7 +549,6 @@ extern char * gc_logfile;
 extern void log_generation_stats(char *logfile, char *header);
 extern void print_generation_stats(void);
 extern double generation_average_age(generation_index_t);
-#define PAGE_INDEX_FMT PRIdPTR
 static inline os_vm_size_t npage_bytes(page_index_t npages)
 {
     gc_assert(npages>=0);
@@ -823,8 +807,6 @@ extern struct fixedobj_page *fixedobj_pages;
 #define fixedobj_page_obj_align(i) (fixedobj_pages[i].attr.parts.obj_align<<WORD_SHIFT)
 #endif
 
-extern page_index_t next_free_page;
-
 extern uword_t
 walk_generation(uword_t (*proc)(lispobj*,lispobj*,uword_t),
                 generation_index_t generation, uword_t extra);
