@@ -182,7 +182,7 @@ if a restart was invoked."
       (is (equal (list (sym "BAZ" "SYM") :internal)
                  (multiple-value-list (sym "BAZ" "SYM")))))))
 
-(with-test (:name :use-package-conflict-set :fails-on :sbcl)
+(with-test (:name :use-package-conflict-set)
   (with-packages (("FOO" (:export "SYM"))
                   ("QUX" (:export "SYM"))
                   ("BAR" (:intern "SYM"))
@@ -254,6 +254,38 @@ if a restart was invoked."
       (is (equal (list (sym "BAZ" "SYM") :internal)
                  (multiple-value-list (sym "BAZ" "SYM")))))))
 
+(with-test (:name :export-conflict.1.only-one-exported)
+  (with-packages (("FOO" (:intern "SYM"))
+                  ("BAR" (:intern "SYM"))
+                  ("BAZ"))
+    (handler-bind ((package-error #'continue))
+      (with-name-conflict-resolution ((sym "BAR" "SYM") :restarted restartedp)
+          (export (list (sym "FOO" "SYM") (sym "BAR" "SYM")) "BAZ")
+        (is restartedp)
+        (is (equal (list (sym "BAR" "SYM") :external)
+                   (multiple-value-list (sym "BAZ" "SYM"))))
+        (let (result)
+          (do-external-symbols (s "BAZ")
+            (push s result))
+          (is (= 1 (length result)))
+          (is (eql (sym "BAR" "SYM") (car result))))))))
+
+(with-test (:name :export-conflict.2.only-one-exported)
+  (with-packages (("FOO" (:intern "SYM"))
+                  ("BAR" (:intern "SYM"))
+                  ("BAZ"))
+    (handler-bind ((package-error #'continue))
+      (with-name-conflict-resolution ((sym "FOO" "SYM") :restarted restartedp)
+          (export (list (sym "FOO" "SYM") (sym "BAR" "SYM")) "BAZ")
+        (is restartedp)
+        (is (equal (list (sym "FOO" "SYM") :external)
+                   (multiple-value-list (sym "BAZ" "SYM"))))
+        (let (result)
+          (do-external-symbols (s "BAZ")
+            (push s result))
+          (is (= 1 (length result)))
+          (is (eql (sym "FOO" "SYM") (car result))))))))
+
 ;;; IMPORT
 (with-test (:name :import-nil.1)
   (with-packages (("FOO" (:use) (:intern "NIL"))
@@ -272,7 +304,7 @@ if a restart was invoked."
       (is (eq 'CL:NIL
               (sym "BAZ" "NIL"))))))
 
-(with-test (:name :import-single-conflict :fails-on :sbcl)
+(with-test (:name :import-single-conflict)
   (with-packages (("FOO" (:export "NIL"))
                   ("BAR" (:export "NIL"))
                   ("BAZ" (:use)))
@@ -373,6 +405,19 @@ if a restart was invoked."
       (when p1 (delete-package p1))
       (when p2 (delete-package p2)))))
 
+(with-test (:name :no-quick-name-conflict-resolution-import-two)
+  (let ((p (make-package "NO-QUICK-NAME-CONFLICT-RESOLUTION-IMPORT-TWO")))
+    (unwind-protect
+         (handler-bind ((name-conflict
+                          (lambda (c)
+                            (assert (not (find-restart 'sb-impl::dont-import-it)))
+                            (assert (not (find-restart 'sb-impl::shadowing-import-it)))
+                            (invoke-restart 'abort))))
+           (restart-case
+               (import (list (make-symbol "FOO") (make-symbol "FOO")) p)
+             (abort ())))
+      (when p (delete-package p)))))
+
 (with-test (:name :quick-name-conflict-resolution-export.1)
   (let (p1 p2)
     (unwind-protect
@@ -435,6 +480,27 @@ if a restart was invoked."
       (when p1 (delete-package p1))
       (when p2 (delete-package p2)))))
 
+(with-test (:name :no-quick-name-conflict-resolution-use-package-two)
+  (let (p1 p2 p3)
+    (unwind-protect
+         (progn
+           (setf p1 (make-package "NO-QUICK-NAME-CONFLICT-RESOLUTION-USE-PACKAGE-TWO.1")
+                 p2 (make-package "NO-QUICK-NAME-CONFLICT-RESOLUTION-USE-PACKAGE-TWO.2")
+                 p3 (make-package "NO-QUICK-NAME-CONFLICT-RESOLUTION-USE-PACKAGE-TWO.3"))
+           (export (intern "FOO" p1) p1)
+           (export (intern "FOO" p2) p2)
+           (handler-bind ((name-conflict
+                            (lambda (c)
+                              (assert (not (find-restart 'sb-impl::keep-old)))
+                              (assert (not (find-restart 'sb-impl::take-new)))
+                              (invoke-restart 'abort))))
+             (restart-case
+                 (use-package (list p1 p2) p3)
+               (abort ()))))
+      (when p1 (delete-package p1))
+      (when p2 (delete-package p2))
+      (when p3 (delete-package p3)))))
+
 (with-test (:name (:package-at-variance-restarts :shadow))
   (let ((p nil)
         (*on-package-variance* '(:error t)))
