@@ -848,3 +848,28 @@
      (let ((seq (make-string 10)))
        (read-sequence seq stream :start 1)))
    (integer 1 10)))
+
+(with-test (:name (replace :or-null))
+  (checked-compile `(lambda (a b)
+                      (declare ((or null simple-base-string) a)
+                               ((or null (simple-array character (*))) b)
+                               (optimize speed))
+                      (replace a b))
+                   :allow-notes nil)
+  (checked-compile `(lambda (a b)
+                      (declare ((or null simple-base-string) a)
+                               ((simple-array character (*)) b)
+                               (optimize speed))
+                      (replace a b))
+                   :allow-notes nil)
+  (checked-compile `(lambda (a b)
+                      (declare ((or null simple-base-string) a b)
+                               (optimize speed))
+                      (replace a b))
+                   :allow-notes nil)
+  (checked-compile `(lambda ()
+                      (coerce nil 'vector))
+                   :allow-notes nil)
+  (checked-compile `(lambda ()
+                      (replace nil #()))
+                   :allow-notes nil))
